* ------------------------------------------------------------------------------
*
*  Model 2 -- Simple multisector CGE model
*
*  Features:   Basic multisector model -- extension of 1-2-3 model
*
*
* ------------------------------------------------------------------------------

$eolcom %

* Model dimensions -------------------------------------------------------------

*  Define SAM dimensions

set is "SAM accounts" /
   AGR      "Agriculture"
   MAN      "Manufacturing"
   SRV      "Services"
   LAB      "Labor account"
   CAP      "Capital account"
   PTX      "Production tax"
   MTX      "Import tax"
   HHD      "Households"
   GOV      "Government"
   INV      "Investment"
   ROW      "Rest of the world"
   TOT      "Row and column sums"
   RES      "Residual"
/ ;

alias(is,js)

set i(is) "Activity and commodity sectors" /
   AGR      "Agriculture"
   MAN      "Manufacturing"
   SRV      "Services"
/ ;

alias(i,j) ;

* Data entry -------------------------------------------------------------------

*  Set values for key elasticities

table keyElast(i,*) "Key model elasticities"

         sigmav    sigmam    omegax
AGR         0.9       3.0       4.0
MAN         0.9       3.0       4.0
SRV         0.9       3.0       4.0
;

parameters
   sigmav(i)   "Capital labor substitution elasticity"
   sigmam(i)   "Armington substitution elasticity"
   omegax(i)   "Domestic vs. export CET elasticity"
;

sigmav(i) = keyElast(i,"sigmav")$(keyElast(i,"sigmav") ne 1)
          + 1.01$(keyElast(i,"sigmav") eq 1) ;

sigmam(i) = keyElast(i,"sigmam")$(keyElast(i,"sigmam") ne 1)
          + 1.01$(keyElast(i,"sigmam") eq 1) ;

omegax(i) = keyElast(i,"omegax") ;

*  Load base SAM
*  Source: GTAP release 8 with adjustments

table sam0(is,js) "Base social accounting matix in $2007 billion"

      AGR   MAN   SRV   LAB   CAP   PTX   MTX   HHD   GOV   INV   ROW   TOT
AGR                                             118               108   177
MAN                                            2207        1139  1598  4944
SRV                                            8380  2579  1867   426 13301
LAB    56  1524  8747                                                 10327
CAP   101   797  3546                                                  4444
PTX    -1    80   577                                                   656
MTX     1    38                                                          39
HHD                   10327  4444                                     14771
GOV                                 656    39  1146                    2579
INV                                            2920  -738         824  3006
ROW    69  2505   382                                                  2956
TOT   177  4944 13301 10327  4444   656    39 14771  2579  3006  2956
;

*  Scale the SAM to trillions of dollars

scalar iscale "Scaling factor for input data"    / 1e-3 / ;
scalar oscale "Scaling factor for output data";
oscale      = 1/iscale ;
sam0(is,js) = iscale*sam0(is,js) ;

* Variable, parameters and equation declarations -------------------------------

*  Declare the variables

variables

*  Endogenous variables under standard closure

   lab(i)      "Labor demand by activity"
   cap(i)      "Capital demand by activity"
   px(i)       "Unit cost"
   pp(i)       "Output price"

   yh          "Household income"
   htax        "Household taxes"
   savh        "Household savings"
   ydh         "Disposable household income"

   xah(i)      "Household demand"
   xag(i)      "Government demand"
   xai(i)      "Investment demand"
   xa(i)       "Armington demand"

   xdd(i)      "Domestic demand for domestic goods"
   xmd(i)      "Import demand"
   pa(i)       "Armington price"

   xds(i)      "Domestic supply to domestic market"
   xes(i)      "Export supply"
   xp(i)       "Output"

   pd(i)       "Price of domestic goods"
   pe(i)       "Price of export goods"
   pm(i)       "User price of imports"

   wage        "Wage rate"
   rent        "Rental rate"

   yg          "Government revenues"
   pg          "Government expenditure deflator"
   ydg         "Government nominal expenditures"
   savg        "Nominal government savings"
   kappa       "Direct tax rate"

   ydi         "Nominal investment"

   walras      "Walras law"

*  Exogenous variables under standard closure

*  Factor supply

   labs        "Labor supply"
   caps        "Capital supply"

*  Policy and fiscal

   ptax(i)     "Production tax"
   mtax(i)     "Tariff rate"
   xg          "Aggregate real government expenditure"
   rsavg       "Real government savings"

*  Savings/investment

   aps         "Household savings propensity"

*  External

   pwm(i)      "World price of imports"
   pwe(i)      "World price of exports"
   savf        "Foreign savings"

*  Model numeraire

   er          "Exchange rate"
;

*  Declare model parameters

parameters

*  Production parameters

   alphal(i)   "Labor share"
   alphak(i)   "Capital share"

*  Domestic absorption share parameters

   alphah(i)   "Household expenditure share parameters"
   alphag(i)   "Government expenditure share parameters"
   alphai(i)   "Investment expenditure share parameters"
   ag          "Government expenditure shifter"

*  Armington parameters

   alphad(i)   "Domestic Armington share"
   alpham(i)   "Imported Armington share"

*  Domestic/export CET parameters

   gammad(i)   "CET domestic share"
   gammae(i)   "CET export share"

*  Define sets for output matrices

sets
   mres "Macro results" /
      wage, rent, yh, htax, savh, ydh,
      ydg, yg, pg, xg, savg, kappa,
      ydi, walras, labs, caps, aps,
      rsavg, savf, er /

   sres "Sectoral results" /
      lab, cap, px, pp,
      xah, xag, xai, xa, xdd, xmd, pa, xds,
      xes, xp, pd, pe, pm,
      ptax, mtax,
      pwe, pwm /

   dmres(mres) "Macro results to be displayed in differences"
      / savg, kappa, walras, aps, rsavg, savf /

   dsres(sres) "Sectoral results to be displated in differences"
      / ptax, mtax /
;

*  Declare matrices that will contain output results

Parameters
   mresMat     "Macro results"
   mpctMat     "Macro percent differences"
   sresMat     "Sector results"
   spctMat     "Sector percent differences"
;

* Equation declaration and model definition ------------------------------------

*  Declare model equations

equations
   labeq(i)    "Labor demand by activity"
   capeq(i)    "Capital demand by activity"
   pxeq(i)     "Unit cost of production"
   ppeq(i)     "Producer price"

   yheq        "Household gross income"
   htaxeq      "Household direct taxes"
   savheq      "Household savings"
   ydheq       "Household expenditures on goods and services"

   xaheq(i)    "Household demand"
   xageq(i)    "Government demand"
   xaieq(i)    "Investment demand"
   xaeq(i)     "Total domestic absorption"

   xddeq(i)    "Demand for domestically produced goods"
   xmdeq(i)    "Import demand"
   paeq(i)     "Price of absorption (Armington)"

   xdseq(i)    "Supply of domestically produced goods to domestic market"
   xeseq(i)    "Export supply"
   xpeq(i)     "Domestic production"

   pdeq(i)     "Equilibrium price of locally produced goods on domestic market"
   peeq(i)     "Producer price of exports"
   pmeq(i)     "User price of imports"

   wageeq      "Labor market equilibrium"
   renteq      "Capital market equilibrium"

   ygeq        "Government revenues"
   pgeq        "Government expenditure deflator"
   ydgeq       "Nominal government expenditure"
   savgeq      "Nominal government savings"
   kappaeq     "Real government savings"

   ydieq       "Nominal investment"
   walraseq    "Balance of payments identity"
;


*  Define model specification

*  Production block

labeq(i)..        % 2.27 Labor demand expression
   lab(i) =e= alphal(i)*xp(i)*(px(i)/wage)**sigmav(i) ;

capeq(i)..        % 2.28 Capital demand expression
   cap(i) =e= alphak(i)*xp(i)*(px(i)/rent)**sigmav(i) ;

pxeq(i)..         % 2.29 Unit cost expression
*  px(i)*xp(i) =e= wage*lab(i) + rent*cap(i) ;
   px(i)**(1-sigmav(i)) =e= alphal(i)*wage**(1-sigmav(i))
                         +  alphak(i)*rent**(1-sigmav(i)) ;

ppeq(i)..         % 2.30 Producer price expression
   pp(i) =e= (1+ptax(i))*px(i) ;

*  Income block

yheq..         % 2.31 Household income expression
   yh =e= sum(i, wage*lab(i) + rent*cap(i)) ;

htaxeq..       % 2.32 Household direct tax expression
   htax =e= kappa*yh ;

savheq..       % 2.33 Household savings expression
   savh =e= aps*(yh - htax) ;

ydheq..        % 2.34 Net disposable income expression
   ydh =e= yh - htax - savh ;

*  Demand block

xaheq(i)$(alphah(i) ne 0)..        % 2.35 Private consumption expression
   xah(i)*pa(i) =e= alphah(i)*ydh ;

xageq(i)$(alphag(i) ne 0)..        % 2.36 Public expenditure expression
   xag(i)*pa(i) =e= alphag(i)*ydg ;

xaieq(i)$(alphai(i) ne 0)..        % 2.37 Investment expenditure expression
   xai(i)*pa(i) =e= alphai(i)*ydi ;

xaeq(i)..         % 2.38 Aggregate Armington demand expression
   xa(i) =e= xah(i) + xag(i) + xai(i) ;

*  Trade block

xddeq(i)..        % 2.39 Domestic demand for domestic prodcution
   xdd(i) =e= alphad(i)*xa(i)*(pa(i)/pd(i))**sigmam(i) ;

xmdeq(i)..        % 2.40 Import demand
   xmd(i) =e= alpham(i)*xa(i)*(pa(i)/pm(i))**sigmam(i) ;

paeq(i)..         % 2.41 Armington price
*  pa(i)*xa(i) =e= pd(i)*xdd(i) + pm(i)*xmd(i) ;
   pa(i)**(1-sigmam(i)) =e= alphad(i)*pd(i)**(1-sigmam(i))
                         +  alpham(i)*pm(i)**(1-sigmam(i)) ;

xdseq(i)..        % 2.42 Domestic supply
   xds(i) =e= gammad(i)*xp(i)*(pd(i)/pp(i))**omegax(i) ;

xeseq(i)..        % 2.43 Export supply
   xes(i) =e= gammae(i)*xp(i)*(pe(i)/pp(i))**omegax(i) ;

xpeq(i)..         % 2.44 Total supply equilibrium
*  pp(i)*xp(i) =e= pd(i)*xds(i) + pe(i)*xes(i) ;
   pp(i)**(1+omegax(i)) =e= gammad(i)*pd(i)**(1+omegax(i))
                         +  gammae(i)*pe(i)**(1+omegax(i)) ;

*  Goods market equilibrium block

pdeq(i)..         % 2.45 Domestic goods market equilibrium
   xdd(i) =e= xds(i) ;

peeq(i)..         % 2.46 Export price expression
   pe(i) =e= er*pwe(i) ;

pmeq(i)..         % 2.47 Import price expression
   pm(i) =e= er*pwm(i)*(1+mtax(i)) ;

*  Factor market equilibrium

wageeq..          % 2.48 Labor market equilibrium
   labs =e= sum(i, lab(i)) ;

renteq..          % 2.49 Capital market equilibrium
   caps =e= sum(i, cap(i)) ;

*  Closure block

ygeq..         % 2.50 Government revenues
   yg =e= sum(i, ptax(i)*px(i)*xp(i) + er*pwm(i)*mtax(i)*xmd(i)) + htax ;

pgeq..         % 2.51 Government expenditure deflator
*  pg*xg =e= sum(i, pa(i)*xag(i)) ;
   pg =e= (1/ag)*prod(i$(alphag(i) ne 0), (pa(i)/alphag(i))**alphag(i)) ;

ydgeq..         % 2.52 Nominal government expenditure
   ydg =e= pg*xg ;

savgeq..       % 2.53 Nominal government savings
   savg =e= yg - ydg ;

kappaeq..      % 2.54 Real government savings
   rsavg*pg =e= savg ;

ydieq..        % 2.55 Nominal investment
   ydi =e= savh + savg + savf*er ;

walraseq..     % 2.56 Balance of payments expression
   walras =e= sum(i, pwe(i)*xes(i) - pwm(i)*xmd(i)) + savf  ;

*  Define model

model cge2 / all / ;
cge2.holdfixed = 1 ;

* Variable initialization ------------------------------------------------------

scalar ifRand / 0 / ;

pp.l(i)    = 1$(ifRand ne 1) + uniform(0.5,20)$(ifRand eq 1) ;
wage.l     = 1$(ifRand ne 1) + uniform(0.5,20)$(ifRand eq 1) ;
rent.l     = 1$(ifRand ne 1) + uniform(0.5,20)$(ifRand eq 1) ;
lab.l(i)   = sam0("lab", i) / wage.l ;
cap.l(i)   = sam0("cap", i) / rent.l ;
xp.l(i)    = (wage.l*lab.l(i) + rent.l*cap.l(i) + sam0("ptx",i)) / pp.l(i) ;
ptax.fx(i) = sam0("ptx",i)/(pp.l(i)*xp.l(i) - sam0("ptx",i)) ;
px.l(i)    = pp.l(i)/(1+ptax.l(i)) ;

yh.l       = sum(i, wage.l*lab.l(i) + rent.l*cap.l(i)) ;
htax.l     = sam0("gov", "hhd") ;
kappa.l    = htax.l/yh.l ;
savh.l     = sam0("inv", "hhd") ;
aps.fx     = savh.l/(yh.l - htax.l) ;
ydh.l      = yh.l - htax.l - savh.l ;

ydg.l      = sum(i, sam0(i, "gov")) ;
ydi.l      = sum(i, sam0(i, "inv")) ;

pa.l(i)    = 1$(ifRand ne 1) + uniform(0.5,20)$(ifRand eq 1) ;
xah.l(i)   = sam0(i, "hhd") / pa.l(i) ;
xag.l(i)   = sam0(i, "gov") / pa.l(i) ;
xai.l(i)   = sam0(i, "inv") / pa.l(i) ;

xa.l(i)    = xah.l(i) + xag.l(i) + xai.l(i) ;

er.fx      = 1$(ifRand ne 1) + uniform(0.5,20)$(ifRand eq 1) ;
pwm.fx(i)  = 1$(ifRand ne 1) + uniform(0.5,20)$(ifRand eq 1) ;
mtax.fx(i) = sam0("mtx",i) / sam0("row", i) ;
pm.l(i)    = er.l*pwm.l(i)*(1+mtax.l(i)) ;

pe.l(i)    = pp.l(i) ;
pd.l(i)    = pp.l(i) ;
pwe.fx(i)  = pe.l(i) / er.l ;

xmd.l(i)   = sam0("row", i) / (er.l*pwm.l(i)) ;
xes.l(i)   = sam0(i, "row") / (er.l*pwe.l(i)) ;

xdd.l(i)   = (pa.l(i)*xa.l(i) - pm.l(i)*xmd.l(i)) / pd.l(i) ;
xds.l(i)   = xdd.l(i) ;

labs.fx    = sum(i, lab.l(i)) ;
caps.fx    = sum(i, cap.l(i)) ;

pg.l       = 1$(ifRand ne 1) + uniform(0.5,20)$(ifRand eq 1) ;
xg.fx      = ydg.l / pg.l ;
savg.l     = sam0("inv", "gov") ;
rsavg.fx   = savg.l / pg.l ;
yg.l       = savg.l + ydg.l ;

savf.fx    = sam0("inv", "row") / er.l ;
walras.l   = 0 ;

* Model calibration ------------------------------------------------------------

alphah(i)  = pa.l(i)*xah.l(i) / ydh.l ;
alphag(i)  = pa.l(i)*xag.l(i) / ydg.l ;
alphai(i)  = pa.l(i)*xai.l(i) / ydi.l ;

ag = (1/pg.l)*prod(i$(alphag(i) ne 0), (pa.l(i)/alphag(i))**alphag(i)) ;

alphal(i)  = (lab.l(i)/xp.l(i))*(wage.l/px.l(i))**sigmav(i) ;
alphak(i)  = (cap.l(i)/xp.l(i))*(rent.l/px.l(i))**sigmav(i) ;

alphad(i)  = (xdd.l(i)/xa.l(i))*(pd.l(i)/pa.l(i))**sigmam(i) ;
alpham(i)  = (xmd.l(i)/xa.l(i))*(pm.l(i)/pa.l(i))**sigmam(i) ;

gammad(i)  = (xds.l(i)/xp.l(i))*(pp.l(i)/pd.l(i))**omegax(i) ;
gammae(i)  = (xes.l(i)/xp.l(i))*(pp.l(i)/pe.l(i))**omegax(i) ;

*  Fix zero activities

xah.fx(i)$(alphah(i) eq 0) = 0 ;
xag.fx(i)$(alphag(i) eq 0) = 0 ;
xai.fx(i)$(alphai(i) eq 0) = 0 ;

* Run shocks -------------------------------------------------------------------

*  Define sets that form the various shocks

sets

   sim       "Set of all simulations" /
      base   "Base year data and initialization"
      check  "Base year simulation with no shocks"
      homog  "Homogeneity shock"
      shock  "Tariff shock" /

   psim(sim) "Simulations that use the solver" / check, homog, shock /
   asim(sim) "Active simulation"
;

asim(sim) = no ;

*  Initialize matrices that will contain model results

mresMat(mres, sim)    = 0 ;
sresMat(sres, i, sim) = 0 ;

*  Select fiscal closure

if(1,

*  Default--fix real government deficit, direct tax rate endogenous

   kappa.lo = -inf ; kappa.up = +inf ;
   rsavg.fx = rsavg.l ;

else

*  Fix direct tax rate, real fiscal deficit endogenous

   rsavg.lo = -inf ; rsavg.up = +inf ;
   kappa.fx = kappa.l ;

) ;

*  Loop over all simulations, introduce shocks and solve except for 'base'

loop(sim,

   asim(sim) = yes ;

   if (asim('homog'),

      er.fx = 2*mresMat('er', 'base') ;

   elseif(ord(sim) gt 1),

      er.fx = mresMat('er', 'base') ;

   ) ;

   if(asim('shock'),

      mtax.fx(i) = 0.05 + 0.01*sresMat('mtax', i, 'base') ;

   elseif(ord(sim) gt 1),

      mtax.fx(i) = 0.01*sresMat('mtax', i, 'base') ;

   ) ;

   if (ord(sim) ne 1,      % solve model except for 'base'

      options limrow=0, limcol=0, solprint=off ;
      solve cge2 using mcp ;

   ) ;

   asim(sim) = no ;

*  Save results -- including base

   sresmat('lab',i,sim)  = oscale*lab.l(i) ;
   sresmat('cap',i,sim)  = oscale*cap.l(i) ;
   sresmat('px',i,sim)   = px.l(i) ;
   sresmat('pp',i,sim)   = pp.l(i) ;
   mresmat('yh',sim)     = oscale*yh.l ;
   mresmat('htax',sim)   = oscale*htax.l ;
   mresmat('savh',sim)   = oscale*savh.l ;
   mresmat('ydh',sim)    = oscale*ydh.l ;
   sresmat('xah',i,sim)  = oscale*xah.l(i) ;
   sresmat('xag',i,sim)  = oscale*xag.l(i) ;
   sresmat('xai',i,sim)  = oscale*xai.l(i) ;
   sresmat('xa',i,sim)   = oscale*xa.l(i) ;
   sresmat('xdd',i,sim)  = oscale*xdd.l(i) ;
   sresmat('xmd',i,sim)  = oscale*xmd.l(i) ;
   sresmat('pa',i,sim)   = pa.l(i) ;
   sresmat('xes',i,sim)  = oscale*xes.l(i) ;
   sresmat('xp',i,sim)   = oscale*xp.l(i) ;
   sresmat('pd',i,sim)   = pd.l(i) ;
   sresmat('pe',i,sim)   = pe.l(i) ;
   sresmat('pm',i,sim)   = pm.l(i) ;
   mresmat('wage',sim)   = wage.l ;
   mresmat('rent',sim)   = rent.l ;
   mresmat('yg',sim)     = oscale*yg.l ;
   mresmat('pg',sim)     = pg.l ;
   mresmat('ydg',sim)    = oscale*ydg.l ;
   mresmat('savg',sim)   = oscale*savg.l ;
   mresmat('kappa',sim)  = 100*kappa.l ;
   mresmat('ydi',sim)    = oscale*ydi.l ;
   mresmat('walras',sim) = oscale*walras.l ;

   mresmat('aps',sim)    = 100*aps.l ;
   sresmat('ptax',i,sim) = 100*ptax.l(i) ;
   sresmat('mtax',i,sim) = 100*mtax.l(i) ;
   mresmat('rsavg',sim)  = oscale*rsavg.l ;
   mresmat('xg',sim)     = oscale*xg.l ;
   mresmat('labs',sim)   = oscale*labs.l ;
   mresmat('caps',sim)   = oscale*caps.l ;
   mresmat('savf',sim)   = oscale*savf.l ;
   sresmat('pwe',i,sim)  = pwe.l(i) ;
   sresmat('pwm',i,sim)  = pwm.l(i) ;
   mresmat('er',sim)     = er.l ;

) ;

mpctMat(mres, psim)$(not dmres(mres) and mresMat(mres, 'base') ne 0)
   = 100*mresMat(mres, psim)/mresMat(mres,'base') - 100 ;
mpctMat(mres, psim)$(dmres(mres) or mresMat(mres, 'base') eq 0)
   = mresMat(mres, psim) - mresMat(mres, 'base') ;

spctMat(sres, i, psim)$(not dsres(sres) and sresMat(sres, i, 'base') ne 0)
   = 100*sresMat(sres, i, psim)/sresMat(sres,i, 'base') - 100 ;
spctMat(sres, i, psim)$(dsres(sres) or sresMat(sres, i, 'base') eq 0)
   = sresMat(sres, i, psim) - sresMat(sres, i, 'base') ;

scalar tolerance / 1e-6 / ;

mpctMat(mres,psim)$(abs(mpctMat(mres,psim)) le tolerance)     = 0 ;
spctMat(sres,i,psim)$(abs(spctMat(sres,i,psim)) le tolerance) = 0 ;
option decimals = 4 ;
display mresMat, mpctMat, sresMat, spctMat ;
